/*

 Album Art Widget for Dashboard.
 http://www.liquidx.net/albumartwidget/

 Copyright (c) 2005, Alastair Tse <alastair@liquidx.net>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright notice, this
 list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

 Neither the Alastair Tse nor the names of its contributors may
 be used to endorse or promote products derived from this software without 
 specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
*/


/* Fetch and Parse Google Images */
var google_base = "http://images.google.com/images";
var google_params = new Array();
google_params["q"] = "";
google_params["hl"] = "en";
google_params["lr"] = ""
google_params["start"] = "";
google_params["sa"] = "N";
google_params["safe"] = "on";
google_params["filter"] = "0";

var google_ajax_base = 'http://ajax.googleapis.com/ajax/services/search/images';
var google_ajax_params = {
  'v': '1.0',
  'callback': 'google_ajax_get_results',
  'imgsz': 'large',
  'q': '',
  'hl': 'en',
  'filter': '1',
  'safe': 'off',
};

var google_image_table = '<table align=center border=0 cellpadding=2 cellspacing=0 width=100%>';
var google_image_content = '<div id=ImgContent>';


/* some global state - inevitable with javascript */

var google_req = {req: null, on_finish: null, on_error: null, url: null};
var google_req_interval = 1000; // 3 seconds delay
var google_req_time = 0; // keep track of the last request, and throttle

var google_throttle_timer = null;
var google_throttle_args = null;

/* constructs a url for album art fetching */

function google_make_url(query) {
    var url = google_base + "?";
    
    google_params["q"] = query;
    
    for (var i in google_params) {
        url = url + i + "=" + encodeURIComponent(google_params[i]) + "&";
    }
    return url;
}

function google_ajax_make_url(query) {
  var url = google_ajax_base + '?';
  google_ajax_params['q'] = query
  for (var arg in google_ajax_params) {
    url = url + arg + '=' + encodeURIComponent(google_ajax_params[arg]) + '&';
  }
  return url;
}

/* handler for when the XmlHttpRequest comes back */

function google_process_request() {
    if (google_req.req.readyState == 4) { 
        google_req_time = (new Date).getTime();
        if (google_req.req.status == 200) {
            if (google_req.on_finish != null) {
                google_req.on_finish(google_req.req);
            }
        }
        else {
            if (google_req.on_error != null) {
                google_req.on_error(google_req.req);
            }
        }
    }
}

/* creates a request - if you execute this faster than the throttle, it will
    either queue your request or silently discard it */

function google_make_request(query, on_finish, on_error) {
    var url = google_make_url(query);

    now = (new Date).getTime();

    if (google_req_time + google_req_interval > now) {
        google_throttle_delay(google_req_interval, google_make_request, query, on_finish, on_error);
        return;
    }
    google_req_time = now;

    if (window.XMLHttpRequest) {
        google_req.req = new XMLHttpRequest();
        google_req.on_finish = on_finish;
        google_req.on_error = on_error;
        google_req.req.url = url;
        google_req.req.onreadystatechange = google_process_request;
        google_req.req.open("GET", url, true);
        google_req.req.send();
    }
}

function google_ajax_make_request(query, on_finish, on_error) { 
  var url = google_ajax_make_url(query);
  now = (new Date).getTime();
  if (google_req_time + google_req_interval > now) {
      google_throttle_delay(google_req_interval, google_ajax_make_request, query, on_finish, on_error);
      return;
  }
  google_req_time = now;    

  // make a json request using json-callback.
  google_req.on_finish = on_finish;
  google_req.on_error = on_error;
  google_req.url = url;
  var script = document.createElement('script');
  script.setAttribute('src', url);
  document.getElementsByTagName('head')[0].appendChild(script);
}

function google_ajax_get_results(response) {
  if (!response || !response.responseData) {
    google_req.on_error(null);
    return;
  }
  
  var results = response.responseData.results;
  
  var urls = [];
  for (var i = 0; i < results.length; i++) {
    var result = results[i];
    if (result['width'] == result['height']) {
      var thumb_url = result['tbUrl'];
      var image_url = result['unescapedUrl'];
      var original_url = result['originalContextUrl'];
      urls.push([thumb_url, image_url]);
    }
  }
  
  if (google_req.on_finish) {
    google_req.on_finish(urls, google_req.url);
  }
}

/* ability to return all the urls returned, and displays them in a DOM
    element called 'images' and a textarea element called 'debug'  */

function google_get_urls(req) {
    var small_large_urls = new Array();
    var is_dyn_img = /dyn\.Img\(\"(.*?)\"\)\;/g;
    
    // NOTE: reponseXML doesn't exist because output is unparsable    
    if (req.responseText.indexOf(google_image_content) != -1) {
        // find image matches
        dyn_img = is_dyn_img.exec(req.responseText);
        while (dyn_img != null) {
            // strip out commas and quotes
            var params = dyn_img[1].split('",');
            var i = 0;
            for (i = 0; i < params.length; i++) {
                params[i] = params[i].slice(1);
            }

            // construct urls
            if (params.length > 6) {
                var img_url = google_base + '?q=tbn:' + params[2] + params[3];
                var a_url = 'http://' + params[3];
                small_large_urls.push([img_url, a_url]);
            }
            
            dyn_img = is_dyn_img.exec(req.responseText);
        }
    }
    return small_large_urls;
}

// --------------------------------------------------------------------------
// private functions you don't have to care about
// --------------------------------------------------------------------------

function google_throttle_delay(delay, fn, query, on_finish, on_error) {

    if (google_throttle_timer != null) {
        clearInterval(google_throttle_timer);
        google_throttle_timer = null;
    }
    google_throttle_args = new Object
    google_throttle_args.fn = fn;
    google_throttle_args.query = query;
    google_throttle_args.on_finish = on_finish;
    google_throttle_args.on_error = on_error;
    google_throttle_timer = setInterval("google_throttle_resume();", delay);    
}

function google_throttle_resume() {
    query = google_throttle_args.query;
    on_finish = google_throttle_args.on_finish;
    on_error = google_throttle_args.on_error;
    fn = google_throttle_args.fn;
    
    if (google_throttle_timer != null) {
        clearInterval(google_throttle_timer);
        google_throttle_timer = null;
    }

    fn(query, on_finish, on_error);
}


// --------------------------------------------------------------------------
// debugging !!!
// --------------------------------------------------------------------------


function test_handler(img_urls, request_url) {
    if ((img_urls == null) || (img_urls.length < 1)) {
        alert('no results');
        return;
    }
    var d = document.getElementById("images");
    for (var i = 0; i < img_urls.length; i++) {
        var img = document.createElement("img");
        img.src = img_urls[i][0];
        d.appendChild(img);
        img = document.createElement("img");
        img.src = img_urls[i][1];
        d.appendChild(img);
        document.getElementById("debug").value += img_urls[i][0];
    }
}


function test_request() {
    var query = document.getElementById("query").value;
    google_ajax_make_request(query, test_handler, alert);
}    


/* To test this, put the code below in an HTML file and play! 
file:///Users/liquidx/Dev/cocoa/AlbumArtWidget/AlbumArt.wdgt/debug.html
<html>
<head>
<script type="text/javascript" src="AlbumArtGoogle.js" charset="utf-8"></script>
</head>
<body>
query: <input type="text" id="query" value="" /><br />
<a href="javascript:test_request();">test</a>
<p><textarea id="debug" cols="80" rows="10"></textarea></p>
<p id="images"></p>
</body>
</html>

*/